/*******************************************************************************
 * Copyright (c) 2015 ALL4TEC.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Jonathan Dumont (ALL4TEC) - initial API and implementation
 ******************************************************************************/
package org.polarsys.esf.core.common.ui.view;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;

import org.eclipse.emf.common.command.BasicCommandStack;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CommandStack;
import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.ui.viewer.IViewerProvider;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain.EditingDomainProvider;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.edit.provider.AdapterFactoryItemDelegator;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.ui.dnd.EditingDomainViewerDropAdapter;
import org.eclipse.emf.edit.ui.dnd.LocalTransfer;
import org.eclipse.emf.edit.ui.dnd.ViewerDragAdapter;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.emf.edit.ui.provider.UnwrappingSelectionProvider;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.part.Page;
import org.eclipse.ui.views.properties.IPropertySourceProvider;
import org.polarsys.esf.core.common.ui.CommonUIActivator;
import org.polarsys.esf.core.common.ui.actions.ViewActionBarContributor;
import org.polarsys.esf.core.common.ui.constants.ImageConstants;
import org.polarsys.esf.core.common.ui.filter.AbstractFilter;
import org.polarsys.esf.core.common.ui.filter.NamePropertyFilter;
import org.polarsys.esf.core.common.ui.view.properties.PropertySourceProvider;

/**
 * Abstract base of all the Pages to use in a PageBookView,
 * with the filters, the editing domain mechanisms, etc.
 * 
 * This abstract page implement the following interfaces :
 * - IEditingDomainProvider : to allow other classes to access to its editing domain
 * - IMenuListener : to be warned when a menu is about to show
 * - IViewerProvider : to allow other classes to access to its viewer
 * - IFiltrePageListener : to enable the filter mechanisms
 * 
 * NB : This kind of page doesn't manage a link with the Properties view, nor a save action
 * as its behaviour is to be dependent of an opened editor. Thus, it's the editor
 * selection with is displayed in the Properties view, and the dirty state is also managed
 * by the editor.
 * 
 * @author $Author: jdumont $
 * @version $Revision: 83 $
 */
public abstract class AbstractPage
    extends Page
    implements IEditingDomainProvider, IMenuListener, IViewerProvider {

    /** The pop-up menu id. */
    private static final String POP_UP_ID = "#PopUp"; //$NON-NLS-1$

    /** The additions menu section id. */
    private static final String ADDITIONS_SEPARATOR_ID = "additions"; //$NON-NLS-1$

    /** The adapter factory used to provide the views of the model elements. */
    private ComposedAdapterFactory mComposedAdapterFactory = null;

    /** The editing domain used to track all the changes to the model elements, if this view is editable. */
    private EditingDomain mEditingDomain = null;

    /**
     * The command stack listener to plug to the command stack of the used editing domain,
     * to be warned the changes happend.
     */
    private CommandStackListener mCommandStackListener = new CommandStackListener() {

        /**
         * {@inheritDoc}
         */
        @Override
        public void commandStackChanged(final EventObject pEvent) {
            AbstractPage.this.getSite().getShell().getDisplay().asyncExec(new Runnable() {

                /**
                 * {@inheritDoc}
                 */
                @Override
                public void run() {
                    handleCommandStackChanged(((CommandStack) pEvent.getSource()).getMostRecentCommand());
                }
            });
        }
    };

    /** The label provider used in the viewer. */
    private IBaseLabelProvider mLabelProvider = null;

    /** The content provider used in the viewer. */
    private IContentProvider mContentProvider = null;

    /** The property source provider used in the viewer. */
    private IPropertySourceProvider mPropertySourceProvider = null;

    /** The view action bar contributor. */
    private ViewActionBarContributor mActionBarContributor = null;

    /** The viewer used in this page. */
    private StructuredViewer mViewer = null;

    /** Flag which indicates if this view is in read only mode. */
    private boolean mReadOnlyMode = true;

    /**
     * Constructor used to specify if the view is read only or not.
     * 
     * @param pReadOnlyMode <code>true</code> if this view is in read only mode, <code>false</code> otherwise
     */
    public AbstractPage(final boolean pReadOnlyMode) {
        super();
        mReadOnlyMode = pReadOnlyMode;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void dispose() {

        // Remove the potential listener on the command stack used
        // This is mandatory as the command stack may not be in this instance context only
        if (mEditingDomain != null && mEditingDomain.getCommandStack() != null) {
            mEditingDomain.getCommandStack().removeCommandStackListener(mCommandStackListener);
        }

        if (mLabelProvider != null) {
            mLabelProvider.dispose();
            mLabelProvider = null;
        }

        if (mContentProvider != null) {
            mContentProvider.dispose();
            mContentProvider = null;
        }

        if (mComposedAdapterFactory != null) {
            mComposedAdapterFactory.dispose();
            mComposedAdapterFactory = null;
        }

        if (mActionBarContributor != null) {
            // Unlink the potential active part registered in the
            // action bar contributor and dispose it
            mActionBarContributor.setActivePart(null);
            mActionBarContributor.dispose();
            mActionBarContributor = null;
        }

        super.dispose();
    }

    /**
     * @return <code>true</code> if the view is in read only mode, <code>false</code> otherwise.
     */
    public boolean isReadOnlyMode() {
        return mReadOnlyMode;
    }

    /**
     * Initialise the editing domain for this page.
     * 
     * The default implementation create a new adapter factory, and
     * a new editing domain for this page instance by calling : {@link AbstractPage#createAdapterFactory()} and
     * {@link AbstractPage#createEditingDomain()}
     */
    protected void initializeEditingDomain() {
        mComposedAdapterFactory = createAdapterFactory();
        mEditingDomain = createEditingDomain();
    }

    /**
     * Create and return an instance of {@link ComposedAdapterFactory} and initialise it with the adapter factories.
     * 
     * @see #getAdapterFactories()
     * @return The created {@link ComposedAdapterFactory}
     */
    protected ComposedAdapterFactory createAdapterFactory() {
        // Create an adapter factory that yields the needed item providers
        final ComposedAdapterFactory vComposedAdapterFactory = new ComposedAdapterFactory();

        // Loop on the adapter factories to add each one to the composed adapter factory
        for (final Class<? extends AdapterFactory> vAdapterFactory : getAdapterFactories()) {
            try {
                vComposedAdapterFactory.addAdapterFactory(vAdapterFactory.newInstance());
            } catch (final InstantiationException e) {
                CommonUIActivator.INSTANCE.log(e);
            } catch (final IllegalAccessException e) {
                CommonUIActivator.INSTANCE.log(e);
            }
        }

        return vComposedAdapterFactory;
    }

    /**
     * Return the adapter factories that will be added to the composed adapter factory
     * used by this page.
     * 
     * @return Adapter factories to add to the composed adapter factory
     */
    protected abstract Collection<Class<? extends AdapterFactory>> getAdapterFactories();

    /**
     * This sets the selection into the active viewer.
     * 
     * @param pSelections The selection elements to set
     * 
     */
    protected void setSelectionToViewer(final Collection<?> pSelections) {
        // Check if this selection is valid
        if (pSelections != null && !pSelections.isEmpty()) {

            // Use a runnable in an UI thread to update the viewer selection
            getSite().getShell().getDisplay().asyncExec(new Runnable() {

                @Override
                public void run() {
                    // Try to select the items in the current content viewer
                    if (mViewer != null) {
                        mViewer.setSelection(new StructuredSelection(pSelections.toArray()), true);
                    }
                }
            });
        }
    }

    /**
     * {@inheritDoc}
     * 
     * This returns the editing domain as required by the {@link IEditingDomainProvider} interface.
     * This is important to implement several mechanisms as the Commands, etc.
     * 
     * @return The current editing domain used
     */
    @Override
    public EditingDomain getEditingDomain() {
        return mEditingDomain;
    }

    /**
     * Create and return an instance of {@link AdapterFactoryEditingDomain}.
     * 
     * This base implementation creates a default editing domain and associate it to a new {@link CommandStack}, and to
     * the resource set returned by the {@link #getResourceSet()} if it's not null. Otherwise, it uses a new resource
     * set.
     * 
     * @return The created instance of {@link EditingDomain}
     */
    protected EditingDomain createEditingDomain() {

        AdapterFactoryEditingDomain vEditingDomain = null;

        // Create a default command stack
        final CommandStack vCommandStack = createCommandStack();

        // Get the resource set used by this page
        final ResourceSet vResourceSet = getResourceSet();

        if (vResourceSet != null) {
            // A resource set is known, use it to create the editing domain
            vEditingDomain = new AdapterFactoryEditingDomain(getAdapterFactory(), vCommandStack, vResourceSet);
            vEditingDomain.setResourceToReadOnlyMap(new HashMap<Resource, Boolean>());

        } else {
            // A resource set is not known, create the editing domain with a new resource set
            vEditingDomain = new AdapterFactoryEditingDomain(
                getAdapterFactory(),
                vCommandStack,
                new HashMap<Resource, Boolean>());
        }

        return vEditingDomain;
    }

    /**
     * Create and return a default command stack, which allow to react to its changes
     * using the {@link AbstractPage#handleCommandStackChanged(Command)} method.
     * 
     * @return The command stack created
     */
    protected BasicCommandStack createCommandStack() {
        // Create the command stack
        final BasicCommandStack vCommandStack = new BasicCommandStack();

        // Add the listener which will notify this page when commands are executed
        addCommandStackListener(vCommandStack);

        return vCommandStack;
    }

    /**
     * Add a custom listener to the given command stack.
     * This listener is used to allow to this page to react to a change in the command stack.
     * 
     * @param pCommandStack The command stack to listen
     */
    protected void addCommandStackListener(final CommandStack pCommandStack) {
        // Add a listener on the command stack to warn the page when something changed
        pCommandStack.addCommandStackListener(mCommandStackListener);
    }

    /**
     * Remove the custom listener to the given command stack.
     * This is mandatory to dispose correctly the page.
     * 
     * @param pCommandStack The command stack to stop to listen
     */
    protected void removeCommandStackListener(final CommandStack pCommandStack) {
        // Remove the potential listener on the command stack
        pCommandStack.removeCommandStackListener(mCommandStackListener);
    }

    /**
     * This method is called when the command stack is changed, to allow the
     * page to react. By default, this will simply select the objects
     * affected by the command in the viewer.
     * 
     * @param pMostRecentCommand The most recent command
     */
    protected void handleCommandStackChanged(final Command pMostRecentCommand) {
        if (pMostRecentCommand != null) {
            // Try to select the affected objects
            setSelectionToViewer(pMostRecentCommand.getAffectedObjects());
        }
    }

    /**
     * {@inheritDoc}
     * 
     * Returns the viewer as required by the {@link IViewerProvider} interface.
     * It can be any kind of viewer (tree, table, etc.).
     * 
     * @return The viewer of this page
     */
    @Override
    public Viewer getViewer() {
        return mViewer;
    }

    /**
     * {@inheritDoc}
     * 
     * By default, return this page viewer control.
     */
    @Override
    public Control getControl() {
        return getViewer().getControl();
    }

    /**
     * Creates a default context menu and set it to the viewer.
     */
    protected void addContextMenu() {
        // Create a default context menu
        final MenuManager vContextMenuManager = new MenuManager();
        vContextMenuManager.add(new Separator(ADDITIONS_SEPARATOR_ID));
        vContextMenuManager.setRemoveAllWhenShown(true);

        // Register this page instance as menu listener to ensure
        // that it will be warn when the menu is opened
        vContextMenuManager.addMenuListener(this);

        // Create a context menu in the new menu manager, and link it to the viewer
        final Menu vMenu = vContextMenuManager.createContextMenu(mViewer.getControl());
        mViewer.getControl().setMenu(vMenu);

        // Register the context menu to the viewer selection
        getSite().registerContextMenu(POP_UP_ID, vContextMenuManager, new UnwrappingSelectionProvider(mViewer));
    }

    /**
     * Add a drag support to the viewer.
     */
    protected void addDragSupport() {
        final int vDndOperations = DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK;
        final Transfer[] vTransfersArray = new Transfer[] {LocalTransfer.getInstance()};

        mViewer.addDragSupport(
            vDndOperations,
            vTransfersArray,
            new ViewerDragAdapter(mViewer));
    }

    /**
     * Add a drop support to the viewer.
     */
    protected void addDropSupport() {
        final int vDndOperations = DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK;
        final Transfer[] vTransfersArray = new Transfer[] {LocalTransfer.getInstance()};

        mViewer.addDropSupport(
            vDndOperations,
            vTransfersArray,
            new EditingDomainViewerDropAdapter(mEditingDomain, mViewer));
    }

    /**
     * Return the adapter factory content provider to associate to the viewer.
     * If no content provider is known, create a default one.
     * 
     * @return Return the content provider to associate to the viewer
     */
    protected IContentProvider getContentProvider() {
        if (mContentProvider == null) {
            // Create a default content provider
            mContentProvider = createContentProvider();
        }

        return mContentProvider;
    }

    /**
     * Create and return a default content provider.
     * The content provider must call {@link #inputChanged(oldInput,newInput) in the method
     * 
     * @return The created content provider
     */
    protected IContentProvider createContentProvider() {
        return new AdapterFactoryContentProvider(getAdapterFactory()) {

            /**
             * {@inheritDoc}
             * 
             * This method is overridden to be able to manage the arrays as input element.
             */
            @Override
            public Object[] getElements(final Object pInputElement) {
                Object[] vElementsArray = new Object[] {};

                // Check if the input element is an array
                if (pInputElement instanceof Object[]) {
                    // If the input is an array, loop on each input object in it
                    // to get its corresponding elements
                    final List<Object> vElementsList = new ArrayList<Object>();
                    for (final Object vInputElement : (Object[]) pInputElement) {
                        vElementsList.addAll(Arrays.asList(super.getElements(vInputElement)));
                    }

                    // Build the final elements array from the list
                    if (!vElementsList.isEmpty()) {
                        vElementsArray = vElementsList.toArray(new Object[vElementsList.size()]);
                    }

                } else {
                    // If the input is a single object, get its element by calling the parent method
                    vElementsArray = super.getElements(pInputElement);
                }

                return vElementsArray;
            }

            /**
             * {@inheritDoc}
             * 
             * This method is overridden to warn the page instance that its input has changed
             * if the content provider input changes.
             */
            @Override
            public void inputChanged(final Viewer pViewer, final Object pOldInput, final Object pNewInput) {
                // Call the parent method
                super.inputChanged(pViewer, pOldInput, pNewInput);

                // Warn the page that its input changed
                AbstractPage.this.inputChanged(pOldInput, pNewInput);
            }

            /**
             * {@inheritDoc}
             * 
             * This method is overridden to warn the page instance that its input has been disposed.
             */
            @Override
            public void dispose() {
                if (viewer != null) {
                    // Set the page input to null
                    AbstractPage.this.inputChanged(viewer.getInput(), null);
                }
                super.dispose();
            }
        };
    }

    /**
     * Called when the input is changed to attach the {@link IEditingDomainProvider} if needed.
     * 
     * @param pOldInput the old input element, or null if the viewer did not previously have an input
     * @param pNewInput the new input element, or null if the viewer does not have an input
     */
    protected void inputChanged(final Object pOldInput, final Object pNewInput) {
        disposeOldInput(pOldInput);
        inputChanged(pNewInput);
    }

    /**
     * React to an input change, to link the page content to the new input.
     * If the input is a resource (or an array of resources), attach all the needed adapters to it.
     * 
     * @param pNewInput The new input value
     */
    private void inputChanged(final Object pNewInput) {

        // Initialise a list of resource to add as input
        List<Resource> vInputResourcesList = new ArrayList<Resource>();

        if (pNewInput instanceof Resource) {
            // If the new input is a resource, add it in the list to manage
            vInputResourcesList.add((Resource) pNewInput);

        } else if (pNewInput instanceof Object[]) {
            // The input is an array, look in its content for the potential resources
            // to use as input
            for (final Object vInput : (Object[]) pNewInput) {
                if (vInput instanceof Resource) {
                    vInputResourcesList.add((Resource) vInput);
                }
            }
        }

        // Finally, loop on the resources to add as input
        for (Resource vResource : vInputResourcesList) {
            // Attach all the needed adapters
            attachAdapterForResource(vResource);

            // Add the resource to the resource set of the used editing domain
            // but lock its edition if the view is read only, and if the editing
            // domain allows it
            if (mReadOnlyMode && mEditingDomain instanceof AdapterFactoryEditingDomain) {
                // Add the resource in the locked part of the resource set
                ((AdapterFactoryEditingDomain) mEditingDomain).getResourceToReadOnlyMap().put(vResource, Boolean.TRUE);

            } else {
                // The view is not in read only, or the editing domain doesn't allow to lock its resource,
                // thus simply add the resource in the resource set
                mEditingDomain.getResourceSet().getResources().add(vResource);
            }
        }
    }

    /**
     * Dispose an old input.
     * 
     * A custom treatment is applied if the input was a resource or an
     * array of resources, to ensure that all the potential listeners or adapters
     * are also disposed.
     * 
     * @param pOldInput The input to dispose
     */
    private void disposeOldInput(final Object pOldInput) {
        if (pOldInput instanceof Resource) {
            // The old input was a resource, detach all the potential adapters
            detachAdapterFromResource((Resource) pOldInput);

        } else if (pOldInput instanceof Object[]) {
            // The old input was an array, check if it contains any resource,
            // to detach all the potential adapters
            for (final Object vOldInput : (Object[]) pOldInput) {
                if (vOldInput instanceof Resource) {
                    detachAdapterFromResource((Resource) vOldInput);
                }
            }
        }
    }

    /**
     * Return the label provider to associate to the viewer.
     * If no label provider is known, create a default one.
     * 
     * @return Return the label provider to associate to the viewer
     */
    protected IBaseLabelProvider getLabelProvider() {
        if (mLabelProvider == null) {
            mLabelProvider = createLabelProvider();
        }
        return mLabelProvider;
    }

    /**
     * Create and return a default label provider based on the adapter factory.
     * 
     * @return The created label provider
     */
    protected IBaseLabelProvider createLabelProvider() {
        return new AdapterFactoryLabelProvider(getAdapterFactory());
    }

    /**
     * Set the input of the viewer.
     * This will be done in an UI thread, and will ensure that the viewer is still available.
     * 
     * @param pInput The new input to set to the viewer
     */
    protected void setViewerInput(final Object pInput) {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {

            /**
             * {@inheritDoc}
             */
            @Override
            public void run() {
                if ((mViewer != null) && !mViewer.getControl().isDisposed()) {
                    synchronized (mViewer) {
                        mViewer.setInput(pInput);
                    }
                }
            }
        });
    }

    /**
     * Return the initial input of the viewer.
     * 
     * @return The initial input of the viewer
     */
    protected abstract Object getInitialInput();

    /**
     * Return the resource set to associate to the editing domain.
     * 
     * If this method return <code>null</code>, a new resource set will be
     * created by default during the editing domain initialisation.
     * 
     * @return The resource set to associate to the editing domain
     */
    protected abstract ResourceSet getResourceSet();

    /**
     * Return the property source provider to associate to the viewer.
     * If no property source provider is known, create a default one.
     * 
     * @return Return the property source provider to associate to the viewer
     */
    protected IPropertySourceProvider getPropertySourceProvider() {
        if (mPropertySourceProvider == null) {
            mPropertySourceProvider = createPropertySourceProvider();
        }
        return mPropertySourceProvider;
    }

    /**
     * Create and return a default property source provider based on the adapter factory.
     * 
     * @return The created property source provider
     */
    protected IPropertySourceProvider createPropertySourceProvider() {
        return new PropertySourceProvider(getAdapterFactory());
    }

    /**
     * Set the message to the status line manager based on the selection count.
     * 
     * @param pSelection The selection to take into account
     */
    protected void updateStatusBar(final ISelection pSelection) {

        String vMessage = null;

        // Ensure that the selection can be parsed
        if (pSelection instanceof IStructuredSelection) {

            // Get the right message, according to the number of elements in the selection
            switch (((IStructuredSelection) pSelection).size()) {
                case 0:
                    vMessage = CommonUIActivator.getMessages().getString("AbstractPage.noobjectselected"); //$NON-NLS-1$
                    break;

                case 1:
                    // Get the text corresponding to the selected object
                    final String vSelectedObjectText = new AdapterFactoryItemDelegator(mComposedAdapterFactory)
                        .getText(((IStructuredSelection) pSelection).getFirstElement());

                    // Build the message with the text found
                    vMessage = CommonUIActivator.getMessages().getString("AbstractPage.oneobjectselected", //$NON-NLS-1$
                        new Object[] {vSelectedObjectText});
                    break;

                default:
                    vMessage = CommonUIActivator.getMessages().getString("AbstractPage.multiobjectsselected", //$NON-NLS-1$
                        new Object[] {Integer.toString(((IStructuredSelection) pSelection).size())});
                    break;
            }
        }

        // Finally update the status message
        setStatusLineMessage(vMessage, false);
    }

    /**
     * Set the message to the status line manager.
     * It can be treated as an error message or not according to the flag given in parameter.
     * 
     * @param pMessage The message to set
     * @param pIsErrorMsg <code>true</code> if the message is an error message, <code>false</code> otherwise
     */
    protected void setStatusLineMessage(final String pMessage, final boolean pIsErrorMsg) {
        // Get the status line manager
        final IStatusLineManager vStatusLineManager = getActionBars().getStatusLineManager();

        if (vStatusLineManager != null) {
            // Update the status message, with an error status or not
            if (pIsErrorMsg) {
                vStatusLineManager.setErrorMessage(ImageConstants.ICON_ERROR, pMessage);
            } else {
                vStatusLineManager.setMessage(ImageConstants.ICON_INFO, pMessage);
            }
        }
    }

    /**
     * {@inheritDoc}
     * 
     * This is overridden to help to fill the context menus with contributions from the Edit menu.
     */
    @Override
    public void menuAboutToShow(final IMenuManager pMenuManager) {
        getActionBarContributor().menuAboutToShow(pMenuManager);
    }

    /**
     * Returns the action bar contributor for this page.
     * If no contributor is known, create a default one.
     * 
     * @return The page action bar contributor
     */
    protected ViewActionBarContributor getActionBarContributor() {
        if (mActionBarContributor == null) {
            mActionBarContributor = createActionBarContributor();
            mActionBarContributor.init(getActionBars(), getSite().getPage());
        }

        return mActionBarContributor;
    }

    /**
     * Update the action bar contributor of this page to use the given part as
     * selection provider, and if possible also as editing domain provider.
     * 
     * This is mandatory to ensure that the contextual actions in the contributor
     * are linked to a selection for their activation and their run.
     * 
     * @param pActivePart The part to use as selection provider and potential editing domain
     * provider for the action bar contributor
     */
    public void updateActionBarContributorActivePart(final IWorkbenchPart pActivePart) {
        // Register the new active part
        getActionBarContributor().setActivePart(pActivePart);

        // Then update the actions
        getActionBarContributor().update();
    }

    /**
     * Create and return a default action bar contributor.
     * 
     * @return The created action bar contributor
     */
    protected ViewActionBarContributor createActionBarContributor() {
        return new ViewActionBarContributor();
    }

    /**
     * Returns the action bars for this page.
     * 
     * @return The {@link IActionBars} of this page
     */
    protected IActionBars getActionBars() {
        return getSite().getActionBars();
    }

    /**
     * This returns the composed adapter factory associated to the viewer.
     * 
     * @return The composed adapter factory associated to the viewer
     */
    protected ComposedAdapterFactory getAdapterFactory() {
        return mComposedAdapterFactory;
    }

    /**
     * Add an editing domain provider adapter to the given resource.
     * 
     * @param pResource The resource to treat
     */
    private void attachAdapterForResource(final Resource pResource) {
        // Check if an editing domain provider is already known for this resource
        // in this case, there is nothing to do
        if (getResourceEditingDomainProvider(pResource) == null) {
            // Create a new editing domain provider and add it to the resource
            final EditingDomainProvider vEditingDomainProvider = new EditingDomainProvider(mEditingDomain);
            pResource.eAdapters().add(vEditingDomainProvider);
        }
    }

    /**
     * Remove the editing domain provider adapter attach to the resource
     * if it has been added only for this page context.
     * 
     * @param pResource The resource to treat
     */
    private void detachAdapterFromResource(final Resource pResource) {
        // Get the editing domain provider linked to the resource
        final IEditingDomainProvider vResourceEditingDomainProvider = getResourceEditingDomainProvider(pResource);
        if (vResourceEditingDomainProvider != null) {

            // Check if the resource editing domain is the same editing domain
            // as the one used in the page. In this case, it has been added explicitly
            // for this page and can thus be removed
            if (mEditingDomain.equals(vResourceEditingDomainProvider.getEditingDomain())) {
                pResource.eAdapters().remove(vResourceEditingDomainProvider);
            }
        }
    }

    /**
     * Return the editing domain provider of a given resource.
     * 
     * @param pResource The resource to treat
     * @return <code>{@link IEditingDomainProvider}</code> found, or null otherwise
     */
    private IEditingDomainProvider getResourceEditingDomainProvider(final Resource pResource) {
        IEditingDomainProvider vEditingDomainProvider = null;

        if (pResource != null) {
            // Get the potential existing editing domain provider
            vEditingDomainProvider = (IEditingDomainProvider) EcoreUtil.getExistingAdapter(
                pResource,
                IEditingDomainProvider.class);
        }

        return vEditingDomainProvider;
    }

    /**
     * {@inheritDoc}
     * 
     * Initialises this page with the given page site.
     * <p>
     * This method is automatically called by the workbench shortly after the part is instantiated. It marks the start
     * of the page's life cycle. Clients must not call this method directly.
     * </p>
     * 
     * @param pSite The page site
     */
    @Override
    public void init(final IPageSite pSite) {
        // Call the parent method
        super.init(pSite);

        // Initialise the editing domain for this page
        initializeEditingDomain();
    }

    /**
     * {@inheritDoc}
     * 
     * Create the controls for this page, and initialise the viewer of this page.
     * 
     * @param pParent The parent control
     */
    @Override
    public void createControl(final Composite pParent) {
        // Initialise the viewer
        initializeViewer(pParent);

        // Add the page styles, menus and toolbars
        addStyles();
        addMenuBarActions();
        addToolBarActions();
    }

    /**
     * This method is used to add the styles to the page : its context menu,
     * and the mechanisms for the drag & drop, and the filters.
     * 
     * User must override this function to add or remove specific styles.
     */
    protected void addStyles() {
        addContextMenu();
        addDragSupport();
        addDropSupport();
        addFilters();
    }

    /**
     * Initialise the viewer of this page.
     * 
     * @param pParent The parent composite
     */
    protected void initializeViewer(final Composite pParent) {
        // Create the viewer and remember it
        mViewer = createViewer(pParent);

        // Set the viewer providers and input
        setViewerProviders();
        setViewerInput(getInitialInput());

        // Register the fact that this page site selection is provided by the viewer
        getSite().setSelectionProvider(mViewer);

        // Finally, add the listeners (double click, etc.) on the viewer
        addViewerListeners();
    }

    /**
     * Set the viewer content and label providers.
     * 
     * This default implementation use the methods {@link AbstractPage#getLabelProvider()} and
     * {@link AbstractPage#getContentProvider()} to find the providers to set.
     */
    protected void setViewerProviders() {
        mViewer.setLabelProvider(getLabelProvider());
        mViewer.setContentProvider(getContentProvider());
    }

    /**
     * Adds the listeners to the viewer.
     * This default implementation add a selection changed and a double click listeners.
     */
    protected void addViewerListeners() {
        addViewerSelectionChangedListener();
        addViewerDoubleCLickListener();
    }

    /**
     * Adds an {@link ISelectionChangedListener} to the viewer.
     */
    protected void addViewerSelectionChangedListener() {
        mViewer.addSelectionChangedListener(new ISelectionChangedListener() {

            /**
             * {@inheritDoc}
             */
            @Override
            public void selectionChanged(final SelectionChangedEvent pEvent) {
                handleSelectionChanged(pEvent);
            }
        });
    }

    /**
     * Adds an {@link IDoubleClickListener} to the viewer.
     */
    protected void addViewerDoubleCLickListener() {
        mViewer.addDoubleClickListener(new IDoubleClickListener() {

            /**
             * {@inheritDoc}
             */
            @Override
            public void doubleClick(final DoubleClickEvent pEvent) {

                // Use an asynchronous UI thread to treat the double click event
                mViewer.getControl().getDisplay().asyncExec(new Runnable() {

                    /**
                     * {@inheritDoc}
                     */
                    @Override
                    public void run() {
                        // Ensure that the viewer is still available
                        if (mViewer != null && !mViewer.getControl().isDisposed()) {
                            handleDoubleClick(pEvent);
                        }
                    }
                });
            }
        });
    }

    /**
     * Called when a double click is made in the viewer.
     * This method does nothing by default.
     * 
     * @param pEvent The {@link DoubleClickEvent}.
     */
    protected void handleDoubleClick(final DoubleClickEvent pEvent) {
        // Nothing to do
    }

    /**
     * Called when the selection changed in the viewer.
     * This method update the status bar message by default.
     * 
     * @param pEvent The {@link ISelectionChangedListener}.
     */
    protected void handleSelectionChanged(final SelectionChangedEvent pEvent) {
        updateStatusBar(pEvent.getSelection());
    }

    /**
     * Create the viewer.
     * This method is abstract as each child will create its own kind of viewer.
     * 
     * @param pParent The composite parent
     * @return The created viewer
     */
    protected abstract StructuredViewer createViewer(Composite pParent);

    /**
     * Refresh the viewer.
     */
    protected void refreshViewer() {
        // Use an asynchronous UI thread to refresh the viewer
        getSite().getShell().getDisplay().asyncExec(new Runnable() {

            /**
             * {@inheritDoc}
             */
            @Override
            public void run() {
                // Ensure that the viewer is still available
                if (mViewer != null && !mViewer.getControl().isDisposed()) {
                    synchronized (mViewer) {
                        mViewer.refresh();
                    }
                }
            }
        });
    }

    /**
     * {@inheritDoc}
     * 
     * Asks this page to take focus within the workbench.
     * Page must assign focus to one of the controls contained in
     * the part's parent composite.
     * 
     * In our case, set the focus to the viewer control by default.
     */
    @Override
    public void setFocus() {
        // Ensure that the viewer is still available
        if (mViewer != null && !mViewer.getControl().isDisposed()) {
            mViewer.getControl().setFocus();
        }
    }

    /**
     * Add an action in the menu bar.
     * 
     * @param pAction The action to add
     */
    protected void addMenuBarAction(final IAction pAction) {
        getActionBars().getMenuManager().add(pAction);
    }

    /**
     * Add an action in the tool bar.
     * 
     * @param pAction The action to add
     */
    protected void addToolBarAction(final IAction pAction) {
        getActionBars().getToolBarManager().add(pAction);
    }

    /**
     * Add a sub-menu to the menu bar.
     * 
     * @param pMenuManager The menu manager to add
     */
    protected void addMenuBarSubMenu(final IMenuManager pMenuManager) {
        getActionBars().getMenuManager().add(pMenuManager);
    }

    /**
     * Add a separator to the menu bar.
     */
    protected void addMenuBarSeparator() {
        getActionBars().getMenuManager().add(new Separator());
    }

    /**
     * Add a separator to the tool bar.
     */
    protected void addToolBarSeparator() {
        getActionBars().getToolBarManager().add(new Separator());
    }

    /**
     * Add the filters returned by {@link #getFilters()} to the viewer.
     */
    protected void addFilters() {
        for (AbstractFilter vFilter : getFilters()) {
            mViewer.addFilter(vFilter);
        }
    }

    /**
     * Called when the filter text changed, to be able to react.
     * 
     * @param pNewFilterText The new filter text to take into account
     */
    public void filterTextChanged(final String pNewFilterText) {
        // Ensure that the viewer is still available
        if (mViewer != null && !mViewer.getControl().isDisposed()) {

            // Loop on the registered filters to apply the new text
            for (final ViewerFilter vViewerFilter : mViewer.getFilters()) {
                if (vViewerFilter instanceof AbstractFilter) {
                    ((AbstractFilter) vViewerFilter).setFilter(pNewFilterText);
                }
            }

            // Finally refresh the viewer content
            refreshViewer();
        }
    }

    /**
     * Return the filters that will be associated to the viewer by {@link #addFilters()}.
     * By default, this returns a table of filters containing a new {@link NamePropertyFilter} filter.
     * 
     * @return The table of filters to associate to the viewer
     */
    protected AbstractFilter[] getFilters() {
        return new AbstractFilter[] {new NamePropertyFilter()};
    }

    /**
     * Add actions to the menu bar.
     * 
     * By default, this method doesn't add any actions, it's only update the menu bar. User can override
     * this method to add actions to the menu bar. User can call at the end of the overriding method
     * super {@link #addMenuBarActions()} to update the menu bar.
     */
    protected void addMenuBarActions() {
        getActionBars().getMenuManager().update(true);
    }

    /**
     * Add actions to the tool bar.
     * 
     * By default, this method doesn't add any actions, it's only update the tool bar. User can override
     * this method to add actions to the tool bar. User can call at the end of the overriding method
     * super {@link #addToolBarActions()} to update the tool bar.
     */
    protected void addToolBarActions() {
        getActionBars().getToolBarManager().update(true);
    }
}
